@@ -1,4 +1,4 @@
-# Copyright (c) 2012 Paul Tagliamonte <paultag@debian.org>
+# Copyright (c) 2013 Paul Tagliamonte <paultag@debian.org>
 #
 # Permission is hereby granted, free of charge, to any person obtaining a
 # copy of this software and associated documentation files (the "Software"),
@@ -224,6 +224,17 @@ class HyASTCompiler(object):
             col_offset=expr.start_column,
             names=[ast.alias(name=str(x), asname=None) for x in expr])
 
+    @builds("import_as")
+    def compile_import_as_expression(self, expr):
+        expr.pop(0)  # index
+        modlist = [expr[i:i + 2] for i in range(0, len(expr), 2)]
+        return ast.Import(
+            lineno=expr.start_line,
+            col_offset=expr.start_column,
+            module=str(expr.pop(0)),
+            names=[ast.alias(name=str(x[0]),
+                             asname=str(x[1])) for x in modlist])
+
     @builds("import_from")
     def compile_import_from_expression(self, expr):
         expr.pop(0)  # index
@@ -247,6 +258,28 @@ class HyASTCompiler(object):
             slice=ast.Index(value=sli),
             ctx=ast.Load())
 
+    @builds("slice")
+    def compile_slice_expression(self, expr):
+        expr.pop(0)  # index
+        val = self.compile(expr.pop(0))  # target
+
+        low = None
+        if expr != []:
+            low = self.compile(expr.pop(0))
+
+        high = None
+        if expr != []:
+            high = self.compile(expr.pop(0))
+
+        return ast.Subscript(
+            lineno=expr.start_line,
+            col_offset=expr.start_column,
+            value=val,
+            slice=ast.Slice(lower=low,
+                            upper=high,
+                            step=None),
+            ctx=ast.Load())
+
     @builds("assoc")
     def compile_assoc_expression(self, expr):
         expr.pop(0)  # assoc
@@ -276,6 +309,37 @@ class HyASTCompiler(object):
         fn.decorator_list = [self.compile(x) for x in expr]
         return fn
 
+    @builds("with_as")
+    def compile_with_as_expression(self, expr):
+        expr.pop(0)  # with-as
+        ctx = self.compile(expr.pop(0))
+        thing = self.compile(expr.pop(0))
+        if isinstance(thing, ast.Tuple):
+            for x in thing.elts:
+                x.ctx = ast.Store()
+
+        thing.ctx = ast.Store()
+
+        ret = ast.With(context_expr=ctx,
+                       lineno=expr.start_line,
+                       col_offset=expr.start_column,
+                       optional_vars=thing,
+                       body=self._mangle_branch([
+                           self.compile(x) for x in expr]))
+
+        if sys.version_info[0] >= 3 and sys.version_info[1] >= 3:
+            ret.items = [ast.withitem(context_expr=ctx, optional_vars=thing)]
+
+        return ret
+
+    @builds(",")
+    def compile_tuple(self, expr):
+        expr.pop(0)
+        return ast.Tuple(elts=[self.compile(x) for x in expr],
+                         lineno=expr.start_line,
+                         col_offset=expr.start_column,
+                         ctx=ast.Load())
+
     @builds("kwapply")
     def compile_kwapply_expression(self, expr):
         expr.pop(0)  # kwapply
@@ -319,6 +383,7 @@ class HyASTCompiler(object):
                            col_offset=e.start_column)
 
     @builds("+")
+    @builds("%")
     @builds("-")
     @builds("/")
     @builds("*")
@@ -330,7 +395,8 @@ class HyASTCompiler(object):
         ops = {"+": ast.Add,
                "/": ast.Div,
                "*": ast.Mult,
-               "-": ast.Sub}
+               "-": ast.Sub,
+               "%": ast.Mod}
 
         inv = expression.pop(0)
         op = ops[inv]
@@ -402,6 +468,10 @@ class HyASTCompiler(object):
             return what
 
         name = self.compile(name)
+        if isinstance(name, ast.Tuple):
+            for x in name.elts:
+                x.ctx = ast.Store()
+
         name.ctx = ast.Store()
 
         return ast.Assign(
@@ -417,6 +487,11 @@ class HyASTCompiler(object):
         expression.pop(0)  # for
         name, iterable = expression.pop(0)
         target = self.compile_symbol(name)
+
+        if isinstance(target, ast.Tuple):
+            for x in target.elts:
+                x.ctx = ast.Store()
+
         target.ctx = ast.Store()
         # support stuff like:
         # (for [x [1 2 3 4]
