@@ -1,4 +1,4 @@
-# Copyright (c) 2012 Paul Tagliamonte <paultag@debian.org>
+# Copyright (c) 2013 Paul Tagliamonte <paultag@debian.org>
 #
 # Permission is hereby granted, free of charge, to any person obtaining a
 # copy of this software and associated documentation files (the "Software"),
@@ -21,6 +21,7 @@
 
 from hy.macros import macro
 from hy.models.expression import HyExpression
+from hy.models.integer import HyInteger
 from hy.models.symbol import HySymbol
 from hy.models.list import HyList
 
@@ -91,3 +92,25 @@ def threading_macro(tree):
         node.insert(1, ret)
         ret = node
     return ret
+
+
+@macro("car")
+@macro("first")
+def first_macro(tree):
+    tree.pop(0)  # "first"
+    ret = tree.pop(0)  # the list
+    # assert tree is empty
+    return HyExpression([HySymbol('get'),
+                         ret,
+                         HyInteger(0)])
+
+
+@macro("cdr")
+@macro("rest")
+def rest_macro(tree):
+    tree.pop(0)  # "first"
+    ret = tree.pop(0)  # the list
+    # assert tree is empty
+    return HyExpression([HySymbol('slice'),
+                         ret,
+                         HyInteger(1)])
